#include <iostream>
#include <string>
#include "Token.h"
#include "Lexer.h"
#include "Parser.h"
#include "Tests/TestLexer.h"
#include "Context.h"

using namespace std;

void test_lexer()
{
    string test_line = "dupa";
    string test_line2 = "def how_many_trees = 1; how_many_trees + 3 == 2; num cut_tree( num how_many) {return how_many -1; != <=}";
    Lexer lexer;
    vector<Token> tokens = lexer.parse_line(test_line);


    for(unsigned int i=0; i< tokens.size(); i++)
    {
        cout << tokens[i].get_value() << " type: " << tokens[i].get_type() << endl;
    }


    tokens = lexer.parse_line(test_line2);

}

void test_parser()
{
    vector<string> lines;
    lines.push_back("def i; bulb; i + 3; string banan = \"kartofel\"; banan = \"banan\"; string kaboom(num how_many_times) { def z; }");
    lines.push_back("num pun");
    lines.push_back("def how_many_trees = 1; how_many_trees + 3 == 2; num cut_tree(num how_many) {return how_many -1}");
    Lexer lexer;
    Context context;
    vector<Token> tokens;

    for(unsigned int i=0; i<lines.size(); i++)
    {
        tokens = lexer.parse_line(lines[i]);
        Parser parser(&context);
        parser.add_tokens(tokens);
        parser.interpret();
        cout << "<<<Parsing number: " << i << " >>>" << endl;
        cout << "Instructions: " << endl ;
        cout << lines[i] << endl << endl;
        cout << parser.report_message;
        cout << parser.error_message << endl;
    }

}

int how_depth_change(vector<Token> tokens)
{
	int change = 0;
	for (unsigned int i = 0; i < tokens.size(); i++)
	{
		if(tokens[i].value == "{") change++;
		else if(tokens[i].value == "}") change--;
	}
	return change;
}

string compute_indentation(int depth_level)
{
	string indent = "";
	for(int i=0; i< depth_level; i++)
	{
		indent += "    ";
	}
	return indent;
}

void interactive()
{
	Lexer lexer;

	Context context;
	Parser parser(&context);

	vector<Token> tokens;

	string input;
	int level_of_depth = 0;
	string indentation = "";
	while(true)
	{
		cout << ">> " + indentation;
		getline(cin, input);
		if(input == "quit()") break;
		tokens = lexer.parse_line(input);

		level_of_depth += how_depth_change(tokens);
		indentation = compute_indentation(level_of_depth);

		parser.add_tokens(tokens);

		if(level_of_depth == 0) {
			parser.interpret();
			parser.program->execute_last();

			//cout << parser.report_message << endl;
			//cout << parser.error_message << endl;
			//cout << parser.show_tokens() << endl;
			//cout << "My tree:\n";
			//cout << parser.program->debug();
			cout << parser.context->debug();
		}
	}

}

int main()
{
    cout << "Sencha-lang interpreter, version 0.12" << endl;

    TestLexer test_l;
    //test_l.run_tests();

    //test_parser();
    //test_lexer();
    interactive();
    return 0;
}
